Shaders = {}

local Shaders_mt = Class(Shaders)

function Shaders.new()
    local instance = setmetatable({}, Shaders_mt)

    instance.shadersFileContent = [==[
<?xml version="1.0" encoding="utf-8" standalone="no" ?>
<CustomShader version="5">
	<!--
	Original Author FS17: 	Martin Fabík ( https://github.com/LoogleCZ/FS17-RealGPSMod )
	Authors FS22: 			Bone2510 & RepiGaming @LSFM - lsfarming-mods.com
	Date: 					06.07.2023
	Version: 				1.0

	History: 				@06.07.2023 - initial implementation in FS 22

	
	Auf diese Datei gilt das Urheberrecht. Die Rechte für die Umsetzung des Mods im LS22 liegen bei LSFM - Bone2510 & Repigaming. Jegliche Nutzung oder konvertierung (auch auszugsweise) in andere 
	LS-Versionen ist ohne eine ausdrückliche Genehmigung der zuvor genannten Personen nicht gestattet!

	-->
	<Parameters>
		<!--
			Player properties.

			Parameter: vec2(x,y) - Player position on plane. Each component in range between 0 and 1
			Parameter: rotation  - Player rotation on plane
			Parameter: playerSize
		-->
		<Parameter name="playerProps" target="playerProps" type="float4" defaultValue="0.5 0.5 3.141586 0.05" minValue="0.0 0.0 -12.5663706144 0.0" maxValue="1.0 1.0 12.5663706144 1.0" />
		<!--
			Map properties.

			Parameter: vec2(x,y) - Map centre
			Parameter: zoom      - Map zoom
			Parameter: rotation  - Map rotation
		-->
		<Parameter name="mapProps" target="mapProps" type="float4" defaultValue="0.5 0.5 1.0 0.0" minValue="0.0 0.0 1.0 -12.5663706144" maxValue="1.0 1.0 50 12.5663706144" />
		<!--
			Screen properties.

			Parameter: vec2(x,y) - Aspect ratio of screen (x axis and y axis)
		-->
		<Parameter name="screenProps" target="screenProps" type="float2" defaultValue="1 1" />
		<Parameter name = "lightControl" target = "lightControl" type = "float"  group="base"   defaultValue = "1.0"              minValue = "-1"             maxValue = "10"/>
	</Parameters>
	<UvUsages>
		<UvUsage textureName="pdaMap" uvType="uv0" uvScale="1.0"/>
		<UvUsage textureName="playerIcon" uvType="uv0" uvScale="1.0"/>
	</UvUsages>
	<Textures>
		<Texture name="pdaMap" defaultColorProfile="sRGB" />
		<Texture name="playerIcon" defaultColorProfile="sRGB" />
	</Textures>
	<Variations>
		<Variation name="PDA_PROJECTION_FLAT">
			<![CDATA[ #define PDA_PROJECTION_FLAT ]]>
		</Variation>
	</Variations>
	<LodLevel startDistance="0">
		<CodeInjections>
			<CodeInjection position = "FS_GLOBALS">
				<![CDATA[
					float  lightControl;
					float4 gEmissiveColor;
				]]>
							</CodeInjection>
			<CodeInjection position="OBJECT_PARAMETERS">
<![CDATA[
	float4 playerProps;
	float4 mapProps;
	float2 screenProps;
	float lightControl;
	]]>
			</CodeInjection>
			<CodeInjection position = "START_FS">
				<![CDATA[ 
				globals.lightControl = object.lightControl;
				globals.gEmissiveColor = float4(1.0,1.0,1.0,1.0);
				]]>
			</CodeInjection>


			<CodeInjection position="SAMPLERS">
<![CDATA[
	sampler2D pdaMap;
	sampler2D playerIcon;
]]>
			</CodeInjection>
			<CodeInjection position="POST_DIFFUSE_COLOR_FS">
<![CDATA[
	float2 albedoPosition = ALBEDO_MAP_TEXCOORDS;
	albedoPosition.x = (albedoPosition.x - 0.5)/object.screenProps.x + 0.5;
	albedoPosition.y = (albedoPosition.y - 0.5)/object.screenProps.y + 0.5;

	/* find position */
	float2 currPos = object.playerProps.xy - albedoPosition;

	float2x2 rotateMap = {
		cos(object.mapProps.w), -sin(object.mapProps.w),
		sin(object.mapProps.w), cos(object.mapProps.w)
	};
	albedoPosition = mul((albedoPosition - 0.5)/object.mapProps.z, rotateMap) + object.mapProps.xy;

	float4 albedo = tex2D(pdaMap, albedoPosition);

	albedo *= step(0.0, albedoPosition.x) * step(0.0, albedoPosition.y) * step(albedoPosition.x, 1.0) * step(albedoPosition.y, 1.0);


	float modifier = step(length(currPos), object.playerProps.w);
	float2x2 rotatePlayer = {
		cos(object.playerProps.z), -sin(object.playerProps.z),
		sin(object.playerProps.z), cos(object.playerProps.z)
	};

	float2 playerTexCoords = mul(currPos, rotatePlayer)/(object.playerProps.w*2) + 0.5;
	float4 albedo2 = tex2D(playerIcon, playerTexCoords);
	albedo = albedo * (1 - modifier) + lerp(albedo, albedo2, albedo2.a) * modifier;

	diffuseColor = albedo;

	float4 mBaseMap = albedo.xyzw;
	globals.gEmissiveColor.xyzw = mBaseMap.xyzw;

	if (object.lightControl<=-1){
		diffuseColor.rgb = float3(0.0, 0.0, 0.0);
		globals.gEmissiveColor.xyzw = float4(0.0, 0.0, 0.0, 0.0);
	}
]]>
			</CodeInjection>
			<CodeInjection position="EMISSIVE_FS">
				<![CDATA[
				#if defined( FRESNEL )
					globals.gEmissiveColor.w *= getFresnel( facing, 0.15 , 4.0 );
				#endif
				// ---------------------------------------------
				if ( In.vs.defaultTexCoords.x > 0.0 ){
				#if defined(EMISSIVE_MAP)
					emissiveColor.rgb *= globals.gEmissiveColor.xyz;
				#else
					// DEFAULT BEHAVIOUR
					emissiveColor.rgb = globals.gEmissiveColor.xyz;
				#endif
				#if defined( UV_GRADIENT )
					emissiveColor.rgb = saturate(In.vs.defaultTexCoords.y*In.vs.defaultTexCoords.y*In.vs.defaultTexCoords.y);
					emissiveColor.rgb *= globals.gEmissiveColor.xyz;
				#endif
					float mLightControl = globals.lightControl;
				#if GPU_PROFILE < GPU_PROFILE_MEDIUM
					mLightControl = saturate(mLightControl);
				#endif
					emissiveColor *= mLightControl;
					emissiveColor *= globals.gEmissiveColor.w;
				}
				// ---------------------------------------------
				]]>
							</CodeInjection>
		</CodeInjections>
	</LodLevel>
</CustomShader>
		]==]

    return instance
end